@@ -17,13 +17,12 @@ import (
 	"golang.org/x/mod/modfile"
 	"golang.org/x/mod/semver"
 	"golang.org/x/sync/errgroup"
-	"golang.org/x/tools/gopls/internal/govulncheck"
 	"golang.org/x/tools/gopls/internal/lsp/command"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/source"
 	"golang.org/x/tools/gopls/internal/span"
+	"golang.org/x/tools/gopls/internal/vulncheck/govulncheck"
 	"golang.org/x/tools/internal/event"
-	"golang.org/x/vuln/osv"
 )
 
 // Diagnostics returns diagnostics from parsing the modules in the workspace.
@@ -61,7 +60,6 @@ func VulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot) (ma
 }
 
 func collectDiagnostics(ctx context.Context, snapshot source.Snapshot, diagFn func(context.Context, source.Snapshot, source.FileHandle) ([]*source.Diagnostic, error)) (map[span.URI][]*source.Diagnostic, error) {
-
 	g, ctx := errgroup.WithContext(ctx)
 	cpulimit := runtime.GOMAXPROCS(0)
 	g.SetLimit(cpulimit)
@@ -192,14 +190,14 @@ func ModVulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot,
 
 	diagSource := source.Govulncheck
 	vs := snapshot.View().Vulnerabilities(fh.URI())[fh.URI()]
-	if vs == nil && snapshot.View().Options().Vulncheck == source.ModeVulncheckImports {
+	if vs == nil && snapshot.Options().Vulncheck == source.ModeVulncheckImports {
 		vs, err = snapshot.ModVuln(ctx, fh.URI())
 		if err != nil {
 			return nil, err
 		}
 		diagSource = source.Vulncheck
 	}
-	if vs == nil || len(vs.Vulns) == 0 {
+	if vs == nil || len(vs.Findings) == 0 {
 		return nil, nil
 	}
 
@@ -208,20 +206,17 @@ func ModVulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot,
 		// must not happen
 		return nil, err // TODO: bug report
 	}
-	type modVuln struct {
-		mod  *govulncheck.Module
-		vuln *govulncheck.Vuln
-	}
-	vulnsByModule := make(map[string][]modVuln)
-	for _, vuln := range vs.Vulns {
-		for _, mod := range vuln.Modules {
-			vulnsByModule[mod.Path] = append(vulnsByModule[mod.Path], modVuln{mod, vuln})
+	vulnsByModule := make(map[string][]*govulncheck.Finding)
+
+	for _, finding := range vs.Findings {
+		if vuln, typ := foundVuln(finding); typ == vulnCalled || typ == vulnImported {
+			vulnsByModule[vuln.Module] = append(vulnsByModule[vuln.Module], finding)
 		}
 	}
-
 	for _, req := range pm.File.Require {
-		vulns := vulnsByModule[req.Mod.Path]
-		if len(vulns) == 0 {
+		mod := req.Mod.Path
+		findings := vulnsByModule[mod]
+		if len(findings) == 0 {
 			continue
 		}
 		// note: req.Syntax is the line corresponding to 'require', which means
@@ -239,10 +234,8 @@ func ModVulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot,
 		// others to 'info' level diagnostics.
 		// Fixes will include only the upgrades for warning level diagnostics.
 		var warningFixes, infoFixes []source.SuggestedFix
-		var warning, info []string
-		var relatedInfo []protocol.DiagnosticRelatedInformation
-		for _, mv := range vulns {
-			mod, vuln := mv.mod, mv.vuln
+		var warningSet, infoSet = map[string]bool{}, map[string]bool{}
+		for _, finding := range findings {
 			// It is possible that the source code was changed since the last
 			// govulncheck run and information in the `vulns` info is stale.
 			// For example, imagine that a user is in the middle of updating
@@ -259,33 +252,42 @@ func ModVulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot,
 			// version in the require statement is equal to or higher than the
 			// fixed version, skip generating a diagnostic about the vulnerability.
 			// Eventually, the user has to rerun govulncheck.
-			if mod.FixedVersion != "" && semver.IsValid(req.Mod.Version) && semver.Compare(mod.FixedVersion, req.Mod.Version) <= 0 {
+			if finding.FixedVersion != "" && semver.IsValid(req.Mod.Version) && semver.Compare(finding.FixedVersion, req.Mod.Version) <= 0 {
 				continue
 			}
-			if !vuln.IsCalled() {
-				info = append(info, vuln.OSV.ID)
-			} else {
-				warning = append(warning, vuln.OSV.ID)
-				relatedInfo = append(relatedInfo, listRelatedInfo(ctx, snapshot, vuln)...)
+			switch _, typ := foundVuln(finding); typ {
+			case vulnImported:
+				infoSet[finding.OSV] = true
+			case vulnCalled:
+				warningSet[finding.OSV] = true
 			}
 			// Upgrade to the exact version we offer the user, not the most recent.
-			if fixedVersion := mod.FixedVersion; semver.IsValid(fixedVersion) && semver.Compare(req.Mod.Version, fixedVersion) < 0 {
+			if fixedVersion := finding.FixedVersion; semver.IsValid(fixedVersion) && semver.Compare(req.Mod.Version, fixedVersion) < 0 {
 				cmd, err := getUpgradeCodeAction(fh, req, fixedVersion)
 				if err != nil {
 					return nil, err // TODO: bug report
 				}
 				sf := source.SuggestedFixFromCommand(cmd, protocol.QuickFix)
-				if !vuln.IsCalled() {
+				switch _, typ := foundVuln(finding); typ {
+				case vulnImported:
 					infoFixes = append(infoFixes, sf)
-				} else {
+				case vulnCalled:
 					warningFixes = append(warningFixes, sf)
 				}
 			}
 		}
 
-		if len(warning) == 0 && len(info) == 0 {
+		if len(warningSet) == 0 && len(infoSet) == 0 {
 			continue
 		}
+		// Remove affecting osvs from the non-affecting osv list if any.
+		if len(warningSet) > 0 {
+			for k := range infoSet {
+				if warningSet[k] {
+					delete(infoSet, k)
+				}
+			}
+		}
 		// Add an upgrade for module@latest.
 		// TODO(suzmue): verify if latest is the same as fixedVersion.
 		latest, err := getUpgradeCodeAction(fh, req, "latest")
@@ -299,11 +301,8 @@ func ModVulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot,
 		if len(infoFixes) > 0 {
 			infoFixes = append(infoFixes, sf)
 		}
-
-		sort.Strings(warning)
-		sort.Strings(info)
-
-		if len(warning) > 0 {
+		if len(warningSet) > 0 {
+			warning := sortedKeys(warningSet)
 			warningFixes = append(warningFixes, suggestRunOrResetGovulncheck)
 			vulnDiagnostics = append(vulnDiagnostics, &source.Diagnostic{
 				URI:            fh.URI(),
@@ -312,10 +311,10 @@ func ModVulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot,
 				Source:         diagSource,
 				Message:        getVulnMessage(req.Mod.Path, warning, true, diagSource == source.Govulncheck),
 				SuggestedFixes: warningFixes,
-				Related:        relatedInfo,
 			})
 		}
-		if len(info) > 0 {
+		if len(infoSet) > 0 {
+			info := sortedKeys(infoSet)
 			infoFixes = append(infoFixes, suggestRunOrResetGovulncheck)
 			vulnDiagnostics = append(vulnDiagnostics, &source.Diagnostic{
 				URI:            fh.URI(),
@@ -350,41 +349,44 @@ func ModVulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot,
 			return vulnDiagnostics, nil // TODO: bug report
 		}
 
-		stdlib := stdlibVulns[0].mod.FoundVersion
-		var warning, info []string
-		var relatedInfo []protocol.DiagnosticRelatedInformation
-		for _, mv := range stdlibVulns {
-			vuln := mv.vuln
-			stdlib = mv.mod.FoundVersion
-			if !vuln.IsCalled() {
-				info = append(info, vuln.OSV.ID)
-			} else {
-				warning = append(warning, vuln.OSV.ID)
-				relatedInfo = append(relatedInfo, listRelatedInfo(ctx, snapshot, vuln)...)
+		var warningSet, infoSet = map[string]bool{}, map[string]bool{}
+		for _, finding := range stdlibVulns {
+			switch _, typ := foundVuln(finding); typ {
+			case vulnImported:
+				infoSet[finding.OSV] = true
+			case vulnCalled:
+				warningSet[finding.OSV] = true
 			}
 		}
-		if len(warning) > 0 {
+		if len(warningSet) > 0 {
+			warning := sortedKeys(warningSet)
 			fixes := []source.SuggestedFix{suggestRunOrResetGovulncheck}
 			vulnDiagnostics = append(vulnDiagnostics, &source.Diagnostic{
 				URI:            fh.URI(),
 				Range:          rng,
 				Severity:       protocol.SeverityWarning,
 				Source:         diagSource,
-				Message:        getVulnMessage(stdlib, warning, true, diagSource == source.Govulncheck),
+				Message:        getVulnMessage("go", warning, true, diagSource == source.Govulncheck),
 				SuggestedFixes: fixes,
-				Related:        relatedInfo,
 			})
+
+			// remove affecting osvs from the non-affecting osv list if any.
+			for k := range infoSet {
+				if warningSet[k] {
+					delete(infoSet, k)
+				}
+			}
 		}
-		if len(info) > 0 {
+		if len(infoSet) > 0 {
+			info := sortedKeys(infoSet)
 			fixes := []source.SuggestedFix{suggestRunOrResetGovulncheck}
 			vulnDiagnostics = append(vulnDiagnostics, &source.Diagnostic{
 				URI:            fh.URI(),
 				Range:          rng,
 				Severity:       protocol.SeverityInformation,
 				Source:         diagSource,
-				Message:        getVulnMessage(stdlib, info, false, diagSource == source.Govulncheck),
+				Message:        getVulnMessage("go", info, false, diagSource == source.Govulncheck),
 				SuggestedFixes: fixes,
-				Related:        relatedInfo,
 			})
 		}
 	}
@@ -392,6 +394,46 @@ func ModVulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot,
 	return vulnDiagnostics, nil
 }
 
+type vulnFindingType int
+
+const (
+	vulnUnknown vulnFindingType = iota
+	vulnCalled
+	vulnImported
+	vulnRequired
+)
+
+// foundVuln returns the frame info describing discovered vulnerable symbol/package/module
+// and how this vulnerability affects the analyzed package or module.
+func foundVuln(finding *govulncheck.Finding) (*govulncheck.Frame, vulnFindingType) {
+	// finding.Trace is sorted from the imported vulnerable symbol to
+	// the entry point in the callstack.
+	// If Function is set, then Package must be set. Module will always be set.
+	// If Function is set it was found in the call graph, otherwise if Package is set
+	// it was found in the import graph, otherwise it was found in the require graph.
+	// See the documentation of govulncheck.Finding.
+	if len(finding.Trace) == 0 { // this shouldn't happen, but just in case...
+		return nil, vulnUnknown
+	}
+	vuln := finding.Trace[0]
+	if vuln.Package == "" {
+		return vuln, vulnRequired
+	}
+	if vuln.Function == "" {
+		return vuln, vulnImported
+	}
+	return vuln, vulnCalled
+}
+
+func sortedKeys(m map[string]bool) []string {
+	ret := make([]string, 0, len(m))
+	for k := range m {
+		ret = append(ret, k)
+	}
+	sort.Strings(ret)
+	return ret
+}
+
 // suggestGovulncheckAction returns a code action that suggests either run govulncheck
 // for more accurate investigation (if the present vulncheck diagnostics are based on
 // analysis less accurate than govulncheck) or reset the existing govulncheck result
@@ -446,66 +488,12 @@ func getVulnMessage(mod string, vulns []string, used, fromGovulncheck bool) stri
 	return b.String()
 }
 
-func listRelatedInfo(ctx context.Context, snapshot source.Snapshot, vuln *govulncheck.Vuln) []protocol.DiagnosticRelatedInformation {
-	var ri []protocol.DiagnosticRelatedInformation
-	for _, m := range vuln.Modules {
-		for _, p := range m.Packages {
-			for _, c := range p.CallStacks {
-				if len(c.Frames) == 0 {
-					continue
-				}
-				entry := c.Frames[0]
-				pos := entry.Position
-				if pos.Filename == "" {
-					continue // token.Position Filename is an optional field.
-				}
-				uri := span.URIFromPath(pos.Filename)
-				startPos := protocol.Position{
-					Line: uint32(pos.Line) - 1,
-					// We need to read the file contents to precisesly map
-					// token.Position (pos) to the UTF16-based column offset
-					// protocol.Position requires. That can be expensive.
-					// We need this related info to just help users to open
-					// the entry points of the callstack and once the file is
-					// open, we will compute the precise location based on the
-					// open file contents. So, use the beginning of the line
-					// as the position here instead of precise UTF16-based
-					// position computation.
-					Character: 0,
-				}
-				ri = append(ri, protocol.DiagnosticRelatedInformation{
-					Location: protocol.Location{
-						URI: protocol.URIFromSpanURI(uri),
-						Range: protocol.Range{
-							Start: startPos,
-							End:   startPos,
-						},
-					},
-					Message: fmt.Sprintf("[%v] %v -> %v.%v", vuln.OSV.ID, entry.Name(), p.Path, c.Symbol),
-				})
-			}
-		}
-	}
-	return ri
-}
-
-func formatMessage(v *govulncheck.Vuln) string {
-	details := []byte(v.OSV.Details)
-	// Remove any new lines that are not preceded or followed by a new line.
-	for i, r := range details {
-		if r == '\n' && i > 0 && details[i-1] != '\n' && i+1 < len(details) && details[i+1] != '\n' {
-			details[i] = ' '
-		}
-	}
-	return strings.TrimSpace(strings.Replace(string(details), "\n\n", "\n\n  ", -1))
-}
-
 // href returns the url for the vulnerability information.
 // Eventually we should retrieve the url embedded in the osv.Entry.
 // While vuln.go.dev is under development, this always returns
 // the page in pkg.go.dev.
-func href(vuln *osv.Entry) string {
-	return fmt.Sprintf("https://pkg.go.dev/vuln/%s", vuln.ID)
+func href(vulnID string) string {
+	return fmt.Sprintf("https://pkg.go.dev/vuln/%s", vulnID)
 }
 
 func getUpgradeCodeAction(fh source.FileHandle, req *modfile.Require, version string) (protocol.Command, error) {
@@ -537,7 +525,7 @@ func SelectUpgradeCodeActions(actions []protocol.CodeAction) []protocol.CodeActi
 	var chosenVersionedUpgrade string
 	var selected []protocol.CodeAction
 
-	seen := make(map[string]bool)
+	seenTitles := make(map[string]bool)
 
 	for _, action := range actions {
 		if strings.HasPrefix(action.Title, upgradeCodeActionPrefix) {
@@ -549,8 +537,8 @@ func SelectUpgradeCodeActions(actions []protocol.CodeAction) []protocol.CodeActi
 			}
 		} else if strings.HasPrefix(action.Title, "Reset govulncheck") {
 			resetAction = action
-		} else if !seen[action.Command.Title] {
-			seen[action.Command.Title] = true
+		} else if !seenTitles[action.Command.Title] {
+			seenTitles[action.Command.Title] = true
 			selected = append(selected, action)
 		}
 	}
